// ============================================================
// Project				:	CityRPG
// Author				:	Iban & Trader & Jookia
// Description			:	Code that overwrites main functions of Blockland.
// ============================================================
// Table of Contents
// 1. Brick Packages
// 2. Client Packages
// 3. Player Packages
// 4. Misc. Packages
// ============================================================

package CityRPG_MainPackage
{
	// ============================================================
	// Section 1 : Brick Packages
	// ============================================================
	function fxDTSBrick::onActivate(%brick, %obj, %client, %pos, %dir)
	{
		Parent::onActivate(%brick, %obj, %client, %pos, %dir);

		%brickData = %brick.getDatablock();

		if(%brickData.CityRPGBrickType == 420)
		{
			if(!%brick.watered)
			{
				%brick.watered = 1;
				messageClient(%client, '', "\c6You watered this\c3" SPC %brickData.uiName SPC "\c6plant.");
				%brick.startGrowing();
			}
			else
			{
				if(%brick.grown)
					%client.centerPrint("\c6This\c3" SPC %brickData.uiName SPC "\c6plant is ready to be harvested!", 1);
				else
					%client.centerPrint("\c6This plant is already watered.", 1);
			}
		}

		if(isObject(%brickData.CityRPGMatchingLot))
		{
			if(!isObject(%client.player.serviceOrigin))
			{
				%client.player.serviceType = "zone";
				%client.player.serviceOrigin = %brick;
				%client.player.serviceFree = %brickData.CityRPGMatchingLot.initialPrice;
				messageClient(%client, '', '\c6It costs \c3%1\c6 to build in this zone. Type \c3/yes\c6 to accept and \c3/no\c6 to decline', %client.player.serviceFee);
			}
			else if(isObject(%client.player.serviceOrigin) && %client.player.serviceOrigin != %brick)
				messageClient(%client, '', "\c6You already have an active transfer. Type \c3/no\c6 to decline it.");
		}	
	}
	
	function fxDTSBrick::onDeath(%brick)
	{
		switch(%brick.getDatablock().CityRPGBrickType)
		{
			case 1:
				%brick.handleCityRPGBrickDelete();
			case 2:
				%brick.handleCityRPGBrickDelete();
			case 3:
				if(getWord($CityRPG::temp::spawnPoints, 0) == %brick)
					$CityRPG::temp::spawnPoints = strReplace($CityRPG::temp::spawnPoints, %brick @ " ", "");
				else
					$CityRPG::temp::spawnPoints = strReplace($CityRPG::temp::spawnPoints, " " @ %brick, "");
		}
		
		parent::onDeath(%brick);
	}

	function servercmdPlantBrick(%client)
	{
		if(!isObject(%client.player.tempBrick))
			return;

		if(%client.isAdmin)
		{
			parent::servercmdPlantBrick(%client);
			return;
		}

		%brick = %client.player.tempBrick;
		%brickData = %brick.getDatablock();

		if(%brickData.CityRPGBrickAdmin)
		{
			commandToClient(%client, 'centerPrint', "You may not place this brick.", 3);
			return;
		}

		if(mFloor(getWord(%brick.rotation, 3)) == 90)
			%boxSize = getWord(%brick.getDatablock().brickSizeY, 1) / 2.5 SPC getWord(%brick.getDatablock().brickSizeX, 0) / 2.5 SPC getWord(%brick.getDatablock().brickSizeZ, 2) / 2.5;
		else
			%boxSize = getWord(%brick.getDatablock().brickSizeX, 1) / 2.5 SPC getWord(%brick.getDatablock().brickSizeY, 0) / 2.5 SPC getWord(%brick.getDatablock().brickSizeZ, 2) / 2.5;
					
		initContainerBoxSearch(%brick.getWorldBoxCenter(), %boxSize, $typeMasks::triggerObjectType);

		while(isObject(%trigger = containerSearchNext()))
		{
			if(%trigger.getDatablock() == CityRPGLotTriggerData.getID())
			{
				%lotTrigger = %trigger;
				break;
			}
		}

		if(!%lotTrigger && %brickData.CityRPGBrickType != 1)
		{
			commandToClient(%client, 'centerPrint', "You cannot plant a brick outside of a lot.\n\c6Use a lot brick to start your build!", 3);
			return;
		}

		if(CityRPGData.getData(%client.bl_id).valueMoney < mFloor(%brick.getDatablock().initialPrice))
		{
			commandToClient(%client, 'centerPrint', "\c6You need at least \c3$" @ mFloor(%brick.getDatablock().initialPrice) SPC "\c6in order to plant this brick!", 3);
			return;
		}

		if(!%lotTrigger && %brickData.CityRPGBrickType == 1 && %client.brickGroup.lotsOwned >= $CityRPG::pref::realestate::maxLots)
		{
			commandToClient(%client, 'centerPrint', "You already own the maximum number of lots.", 3);
			return;
		}

		if(%lotTrigger && %brick.getDatablock().CityRPGBrickType != 1)
		{
			%lotTriggerMinX = getWord(%lotTrigger.getWorldBox(), 0);
			%lotTriggerMinY = getWord(%lotTrigger.getWorldBox(), 1);
			%lotTriggerMinZ = getWord(%lotTrigger.getWorldBox(), 2);
						
			%lotTriggerMaxX = getWord(%lotTrigger.getWorldBox(), 3);
			%lotTriggerMaxY = getWord(%lotTrigger.getWorldBox(), 4);
			%lotTriggerMaxZ = getWord(%lotTrigger.getWorldBox(), 5);
						
			%brickMinX = getWord(%brick.getWorldBox(), 0) + 0.0016;
			%brickMinY = getWord(%brick.getWorldBox(), 1) + 0.0013;
			%brickMinZ = getWord(%brick.getWorldBox(), 2) + 0.00126;
						
			%brickMaxX = getWord(%brick.getWorldBox(), 3) - 0.0016;
			%brickMaxY = getWord(%brick.getWorldBox(), 4) - 0.0013;
			%brickMaxZ = getWord(%brick.getWorldBox(), 5) - 0.00126;

			if(%brickMinX < %lotTriggerMinX || %brickMinY < %lotTriggerMinY || %brickMinZ < %lotTriggerMinZ || %brickMaxX > %lotTriggerMaxX || %brickMaxY > %lotTriggerMaxY || %brickMaxZ > %lotTriggerMaxZ)
			{
				CommandToClient(%client, 'ServerMessage', 'MsgPlantError_Unstable');
				return;
			}
		}

		if(%lotTrigger && %brickData.CityRPGBrickType == 1)
		{
			commandToClient('centerPrint', "You can not place a lot within a lot.", 3);
			return;
		}

		if(%lotTrigger && getTrustLevel(%lotTrigger.parent, %client.player) < 1)
		{
			commandToClient(%client, 'centerPrint', (getBrickGroupFromObject(%lotTrigger.parent).name $= "") ? "\c1BL_ID:" SPC getBrickGroupFromObject(%lotTrigger.parent).bl_id : getBrickGroupFromObject(%lotTrigger.parent).name SPC "\c0does not trust you enough to do that.", 1);
			return;
		}

		if(%lotTrigger && %brickData.getID() == brickVehicleSpawnData.getID() && CityRPGData.getData(%client.bl_id).valueMoney < mFloor($CityRPG::prices::vehicleSpawn))
		{
			commandToClient(%client, 'centerPrint', "\c6You need at least \c3$" @ mFloor($CityRPG::prices::vehicleSpawn) SPC "\c6in order to plant this vehicle spawn!", 3);
			return;
		}
			
		if(%lotTrigger && %brickData.CityRPGBrickType == 420)
		{
			for(%a = 0; %a < %client.brickGroup.getCount(); %a++)
				if(%client.brickGroup.getObject(%a).getDatablock().CityRPGBrickType == 420)
					%drugsOwned += 1;

			if(%drugsOwned >= $CityRPG::Drugs::DrugLimit)
			{
				commandToClient(%client, 'centerPrint', "\c6You have hit the limit of drugs you are allowed to place.", 3);
				return;
			}
		}

		Parent::servercmdPlantBrick(%client);
	}

	function fxDTSBrick::onPlant(%brick)
	{
		Parent::onPlant(%brick);

		if(!%brick.isPlanted || !isObject(%brick) || !isObject(%client = %brick.getGroup().client))
			return;

		%brickData = %brick.getDatablock();

		if((%price = %brick.getDatablock().initialPrice) <= CityRPGData.getData(%client.bl_id).valueMoney && %price && !%client.isAdmin)
		{
			messageClient(%client, '', "\c6You have paid \c3$" @ mFloor(%brick.getDatablock().initialPrice) @ "\c6 to plant this brick.");
			CityRPGData.getData(%client.bl_id).valueMoney -= mFloor(%brick.getDatablock().initialPrice);
			%client.setInfo();
		}

		switch(%brick.getDatablock().CityRPGBrickType)
		{
			case 1:
				%brick.schedule(1, "handleCityRPGBrickCreation");
			case 2:
				%brick.schedule(1, "handleCityRPGBrickCreation");
			case 3:
				$CityRPG::temp::spawnPoints = ($CityRPG::temp::spawnPoints $= "") ? %brick : $CityRPG::temp::spawnPoints SPC %brick;
			case 4:
				%seed = getRandom(1, ResourceSO.treeCount);
				%brick.id = ResourceSO.tree[%seed].id;
				%brick.BPH = ResourceSO.tree[%seed].BPH;
				%brick.name = ResourceSO.tree[%seed].name;
				%brick.totalHits = ResourceSO.tree[%seed].totalHits;
				%brick.color = getClosestPaintColor(ResourceSO.tree[%seed].color);
				%brick.setColor(%brick.color);
			case 5:
				%seed = getRandom(1, ResourceSO.mineralCount);
				%brick.id = ResourceSO.mineral[%seed].id;
				%brick.BPH = ResourceSO.mineral[%seed].BPH;
				%brick.name = ResourceSO.mineral[%seed].name;
				%brick.totalHits = ResourceSO.mineral[%seed].totalHits;
				%brick.color = getClosestPaintColor(ResourceSO.mineral[%seed].color);
				%brick.setColor(%brick.color);
			case 420:
				%brick.harvestMin = $CityRPG::Drugs::HarvestMin[%brickData.uiName];
				%brick.harvestMax = $CityRPG::Drugs::HarvestMax[%brickData.uiName];
				%brick.growthTime = $CityRPG::Drugs::GrowthTime[%brickData.uiName];
				%brick.grown = 0;
				%brick.watered = 0;
				%brick.currentColor = 45;
				%brick.setColor(45);
				%brick.growTime = 0;
				%brick.health = 0;
				%brick.random = getRandom($CityRPG::Drugs::HarvestMin[%brickData.uiName], $CityRPG::Drugs::HarvestMax[%brickData.uiName]);
			default:
				if(%brick.getDatablock().getID() == brickVehicleSpawnData.getID() && !%client.isAdmin)
				{
					commandToClient(%client, 'centerPrint', "\c6You have paid \c3$" @ mFloor($CityRPG::prices::vehicleSpawn) @ "\c6 to plant this vehicle spawn.", 3);
					CityRPGData.getData(%client.bl_id).valueMoney -= mFloor($CityRPG::prices::vehicleSpawn);
					%client.setInfo();
				}		
		}
	}

	function fxDTSBrick::onLoadPlant(%brick)
	{		
		Parent::onLoadPlant(%brick);
	
		if(!%brick.isPlanted)
			return;

		switch(%brick.getDatablock().CityRPGBrickType)
		{
			case 1:
				%brick.schedule(1, "handleCityRPGBrickCreation");
			case 2:
				%brick.schedule(1, "handleCityRPGBrickCreation");
			case 3:
				$CityRPG::temp::spawnPoints = ($CityRPG::temp::spawnPoints $= "") ? %brick : $CityRPG::temp::spawnPoints SPC %brick;
			case 4:
				%seed = getRandom(1, ResourceSO.treeCount);
				%brick.id = ResourceSO.tree[%seed].id;
				%brick.BPH = ResourceSO.tree[%seed].BPH;
				%brick.name = ResourceSO.tree[%seed].name;
				%brick.totalHits = ResourceSO.tree[%seed].totalHits;
				%brick.color = getClosestPaintColor(ResourceSO.tree[%seed].color);
				%brick.setColor(%brick.color);
			case 5:
				%seed = getRandom(1, ResourceSO.mineralCount);
				%brick.id = ResourceSO.mineral[%seed].id;
				%brick.BPH = ResourceSO.mineral[%seed].BPH;
				%brick.name = ResourceSO.mineral[%seed].name;
				%brick.totalHits = ResourceSO.mineral[%seed].totalHits;
				%brick.color = getClosestPaintColor(ResourceSO.mineral[%seed].color);
				%brick.setColor(%brick.color);
			case 420:
				%brick.harvestMin = $CityRPG::Drugs::HarvestMin[%brickData.uiName];
				%brick.harvestMax = $CityRPG::Drugs::HarvestMax[%brickData.uiName];
				%brick.growthTime = $CityRPG::Drugs::GrowthTime[%brickData.uiName];
				%brick.grown = 0;
				%brick.watered = 0;
				%brick.currentColor = 45;
				%brick.setColor(45);
				%brick.growTime = 0;
				%brick.health = 0;
				%brick.random = getRandom($CityRPG::Drugs::HarvestMin[%brickData.uiName], $CityRPG::Drugs::HarvestMax[%brickData.uiName]);
				%brick.setEmitter("None");
		}
	}

	function fxDTSBrick::onRemove(%brick)
	{
		switch(%brick.getDatablock().CityRPGBrickType)
		{
			case 1:
				%brick.handleCityRPGBrickDelete();
			case 2:
				%brick.handleCityRPGBrickDelete();
			case 3:
				if(getWord($CityRPG::temp::spawnPoints, 0) == %brick)
					$CityRPG::temp::spawnPoints = strReplace($CityRPG::temp::spawnPoints, %brick @ " ", "");
				else
					$CityRPG::temp::spawnPoints = strReplace($CityRPG::temp::spawnPoints, " " @ %brick, "");
		}
		
		parent::onRemove(%brick);
	}

	function fxDTSBrick::setEmitter(%brick, %emitter, %client)
	{
		%brickData = %brick.getDatablock();

		if(%brickData.CityRPGBrickType == 420 && %emitter !$= "None")
			%emitter = %brick.drugEmitter.getID();

		parent::setEmitter(%brick, %emitter, %client);
	}
	
	function fxDTSBrick::setColor(%brick, %color)
	{
		%brickData = %brick.getDatablock();

		if(%brickData.CityRPGBrickType == 4 || %brickData.CityRPGBrickType == 5 || %brickData.CityRPGBrickType == 420)
			%color = %brick.color;

		parent::setColor(%brick, %color);
	}

	function fxDTSBrick::setVehicle(%brick, %vehicle)
	{
		if(%brick.getDatablock().getName() !$= "CityRPGPoliceVehicleData")
		{
			if(!isObject(%brick.getGroup().client) || !%brick.getGroup().client.isAdmin)
			{
				if(isObject(%vehicle))
				{
					for(%a = 0; $CityRPG::vehicles::banned[%a] !$= "" && !%hasBeenBanned; %a++)
					{
						if(%vehicle.getName() $= $CityRPG::vehicles::banned[%a])
						{
							if(isObject(%brick.getGroup().client))
							{
								messageClient(%brick.getGroup().client, '', "\c6Standard users may not spawn a\c3" SPC %vehicle.uiName @ "\c6.");
							}
							%vehicle = 0;
							%hasBeenBanned = true;
						}
					}
				}
			}
		}
		
		parent::setVehicle(%brick, %vehicle);
	}
	
	function fxDTSBrick::setItem(%brick, %datablock, %client)
	{
		if(!%brick.getDatablock().CityRPGPermaspawn && %brick != $LastLoadedBrick)
		{
			if(!isObject(%brick.item) || %brick.item.getDatablock() != %datablock)
			{
				%ownerBG = getBrickGroupFromObject(%brick);
				
				if(%ownerBG.client.isAdmin)
					parent::setItem(%brick, %datablock, %client);
			}
			else
				parent::setItem(%brick, %datablock, %client);
		}
		else
			parent::setItem(%brick, %datablock, %client);
	}
	
	function fxDTSBrick::setMusic(%brick, %song)
	{
		if(%brick.getGroup().client.isAdmin || %brick == $LastLoadedBrick)
			parent::setMusic(%brick, %song);
		else
			parent::setMusic(%brick, 0);
	}
	
	function fxDTSBrick::spawnItem(%brick, %pos, %datablock, %client)
	{
		if(isObject(%owner = getBrickGroupFromObject(%brick).client) && %owner.isAdmin)
			parent::spawnItem(%brick, %pos, %datablock, %client);
	}
	
	function fxDTSBrick::respawnVehicle(%brick, %client)
	{
		if(%brick.getDatablock().getName() $= "CityRPGPoliceVehicleData")
		{
			%stars = CityRPG_GetMaxStars();
			if(%stars == 6)
				%brick.setVehicle(tankVehicle);
			else if(%stars >= 3)
				%brick.setVehicle(blockoCarVehicle);
			else
				%brick.setVehicle(horseArmor);
		}
			
		parent::respawnVehicle(%brick, %client);
	}
	
	// ============================================================
	// Section 2 : Client Packages
	// ============================================================
	function gameConnection::onClientEnterGame(%client)
	{
		if(!isObject(CityRPGData.getData(%client.bl_id)))
			CityRPGData.addData(%client.bl_id);

		if(!isObject(CityRPGData.user[%client.bl_id]))
			CityRPGData.user[%client.bl_id] = CityRPGData.getData(%client.bl_id);
		
		parent::onClientEnterGame(%client);
		
		if(isObject(CityRPGMini))
			CityRPGMini.addMember(%client);
		else
			CityRPG_BuildMinigame();

		if(CityRPGData.getData(%client.bl_id).valueGender $= "ambig")
		{
			messageClient(%client, '', "\c6You have been set as a male by default. Type \c3/sexChange\c6 to be known as female.");
			CityRPGData.getData(%client.bl_id).valueGender = "Male";
		}

		%client.gender = CityRPGData.getData(%client.bl_id).valueGender;
		
		%client.doIPKTest();

		%client.schedule(1000,applyForcedBodyParts);
		%client.schedule(1000,applyForcedBodyColors);
	}
	
	function gameConnection::onClientLeaveGame(%client)
	{
		if(%client.getWantedLevel())
			if(!isObject(%client.CityRPGLotBrick) || getBrickGroupFromObject(%client.CityRPGLotBrick).client != %client)
				%client.arrest();

		if(isObject(%client.player) && !getWord(CityRPGData.getData(%client.bl_id).valueJailData, 1))
		{
			for(%a = 0; %a < %client.player.getDatablock().maxTools; %a++)
			{
				%tool = %client.player.tool[%a];
				
				if(isObject(%tool))
				{
					%tool = %tool.getName();
					%tools = (%tools !$= "" ? %tools SPC %tool : %tool);
				}
			}
			
			if(%tools !$= "")
				CityRPGData.getData(%client.bl_id).valueTools = %tools;
			else
				error("No Tool String!");
		}
		
		parent::onClientLeaveGame(%client);
	}
	
	function gameConnection::spawnPlayer(%client)
	{
		%client.applyForcedBodyColors();
		%client.applyForcedBodyParts();
		
		parent::spawnPlayer(%client);
		
		if(%client.moneyOnSuicide > 0)
		{
			CityRPGData.getData(%client.bl_id).valueMoney = %client.moneyOnSuicide;
		}
		
		%client.hasBeenDead = 0;
		%client.moneyOnSuicide = 0;
		
		%client.player.setScale("1 1 1");
		%client.player.setDatablock(%client.getJobSO().db);
		%client.player.giveDefaultEquipment();
		%client.SetInfo();
	}
	
	function gameConnection::onDeath(%client, %killerPlayer, %killer, %damageType, %unknownA)
	{
		if(!getWord(CityRPGData.getData(%client.bl_id).valueJailData, 1))
		{
			if(%client.player.currTool)
				serverCmddropTool(%client, %client.player.currTool);
		}
		
		if(isObject(%client.CityRPGTrigger))
			%client.CityRPGTrigger.getDatablock().onLeaveTrigger(%client.CityRPGTrigger, %client.player);
		
		if(isObject(%killer) && %killer != %client)
		{
			if(CityRPGData.getData(%client.bl_id).valueBounty > 0)
			{
				if(!%killer.getJobSO().bountyClaim)
				{
					commandToClient(%killer, 'centerPrint', "\c6You have commited a crime. [\c3Claiming a Hit\c6]", 1);
					CityRPG_AddDemerits(%killer.bl_id, $CityRPG::demerits::bountyClaiming);
				}
				
				messageClient(%killer, '', "\c6Hit was completed successfully. The money has been wired to your bank account.");
				CityRPGData.getData(%killer.bl_id).valueBank += CityRPGData.getData(%client.bl_id).valueBounty;
				CityRPGData.getData(%client.bl_id).valueBounty = 0;
			}
			else if(CityRPG_illegalAttackTest(%killer, %client))
			{
				if(%killer.lastKill + 15 >= $sim::time)
				{
					commandToClient(%killer, 'centerPrint', "\c6You have commited a crime. [\c3Killing Spree\c6]", 1);
					CityRPG_AddDemerits(%killer.bl_id, ($CityRPG::demerits::murder * 1.5));
				}
				else
				{
					commandToClient(%killer, 'centerPrint', "\c6You have commited a crime. [\c3Murder\c6]", 1);
					CityRPG_AddDemerits(%killer.bl_id, $CityRPG::demerits::murder);
				}
				%killer.lastKill = $sim::time;
			}
		}
		
		CityRPGData.getData(%client.bl_id).valueTools = "";
		parent::onDeath(%client, %player, %killer, %damageType, %unknownA);
	}
	
	function gameConnection::setScore(%client, %score)
	{
		%score = CityRPGData.getData(%client.bl_id).valueMoney + CityRPGData.getData(%client.bl_id).valueBank;
		parent::setScore(%client, %score);
	}
	
	// ==================================s==========================
	// Section 3 : Player Packages
	// ============================================================	
	function player::activateStuff(%this)
	{
		parent::activateStuff(%this);
		%client = %this.client;
		if(%client.getJobSO().thief)
		{
			%target = containerRayCast(%this.getEyePoint(), vectorAdd(vectorScale(vectorNormalize(%this.getEyeVector()), 2.5), %this.getEyePoint()), $TypeMasks::All);
			
			if(%this.lastPickpocket + 5 <= $sim::time && %this.client != %target.client && isObject(%target.client) && %target.getClassName() $= "Player" && getWord(CityRPGData.getData(%this.client.bl_id).valueJailData, 1) < 1 && getWord(CityRPGData.getData(%target.client.bl_id).valueJailData, 1) < 1)
			{
				if(%target.client.getJobSO().thief || %this.client.getWantedLevel())
				{
					messageClient(%this.client, '', "\c6Your target seems very aware of what you're doing...");
				}
				else
				{
					if(CityRPGData.getData(%target.client.bl_id).valueMoney > 0)
					{				
						%this.lastPickpocket = $sim::time;
						
						%money = CityRPGData.getData(%target.client.bl_id).valueMoney;
						
						if(%money >= 100)
							%maxValue = 6;
						else if(%money >= 50)
							%maxValue = 5;
						else if(%money >= 20)
							%maxValue = 4;
						else if(%money >= 10)
							%maxValue = 3;
						else if(%money >= 5)
							%maxValue = 2;
						else
							%maxValue = 1;
								
						%billStolen = mFloor(getRandom(1, %maxValue));
						
						switch(%billStolen)
						{
							case 1: %theft = 1;
							case 2: %theft = 5;
							case 3: %theft = 10;
							case 4: %theft = 20;
							case 5: %theft = 50;
							case 6: %theft = 100;
						}
						
						CityRPGData.getData(%this.client.bl_id).valueMoney += %theft;
						CityRPGData.getData(%target.client.bl_id).valueMoney -= %theft;
						
						%pRotate = getWord(%this.rotation, 3);
						%tRotate = getWord(%target.rotation, 3);
						
						messageClient(%this.client, '', "\c6You have stolen a \c4$" @ %theft @ "\c6 bill from\c4" SPC  %target.client.name @ "\c6.");
						
						if(%tRotate + 45 < %pRotate || %tRotate - 45 > %pRotate)
						{
							if(CityRPGData.getData(%this.client.bl_id).valueDemerits + $CityRPG::demerits::pickpocketing < $CityRPG::pref::demerits::wantedLevel)
							{
								%demerits = $CityRPG::pref::demerits::wantedLevel;
								serverCmdmessageSent(%target.client, "Police!" SPC %this.client.name SPC "is a thief!");
							}
							else
								%demerits = $CityRPG::demerits::pickpocketing;
							
							commandToClient(%this.client, 'centerPrint', "\c6You have commited a crime. [\c4Pickpocketing\c6]", 3);
							CityRPG_AddDemerits(%this.client.bl_id, %demerits);
							
							serverCmdAlarm(%target.client);
							messageClient(%target.client, '', "\c6You have been pick-pocketed by \c4" @ (%this.client.getWantedLevel() ? %this.client.name : "an unknown thief") @ "\c6!");
						}
						
						%this.client.SetInfo();
						%target.client.SetInfo();
					}
					else
					{
						%this.lastPickpocket = $sim::time;
						%pRotate = getWord(%this.rotation, 3);
						%tRotate = getWord(%target.rotation, 3);
						
						messageClient(%this.client, '', "\c6They have no money!");
						
						if(%tRotate + 45 < %pRotate || %tRotate - 45 > %pRotate)
						{
							if(CityRPGData.getData(%this.client.bl_id).valueDemerits + $CityRPG::demerits::pickpocketing < $CityRPG::pref::demerits::wantedLevel)
							{
								%demerits = $CityRPG::pref::demerits::wantedLevel;
								serverCmdAlarm(%target.client);
								messageClient(%target.client, '', "\c6You have been pick-pocketed by \c4" @ (%this.client.getWantedLevel() ? %this.client.name : "an unknown thief") @ "\c6!");
							}
							else
								%demerits = ($CityRPG::demerits::pickpocketing / 2);
							
							commandToClient(%this.client, 'centerPrint', "\c6You have commited a crime. [\c4Attempted Pickpocketing\c6]", 3);
							CityRPG_AddDemerits(%this.client.bl_id, %demerits);
							%this.client.SetInfo();
						}
					}
				}
			}
		}
	}

	function player::mountImage(%this, %datablock, %slot)
	{		
		if(!getWord(CityRPGData.getData(%this.client.bl_id).valueJailData, 1) || $CityRPG::demerits::jail::image[%datablock.getName()])
			parent::mountImage(%this, %datablock, %slot);
		else
			%this.playthread(2, root);
	}
	
	function player::damage(%this, %obj, %pos, %damage, %damageType)
	{
		if(%this.kevlar && getword(%pos, 2) <= getword(%this.getWorldBoxCenter(), 2) - 3.3)
			%damage /= 2;
		
		if(isObject(%obj.client) && isObject(%this.client) && isObject(%this))
		{
			if(%obj.getDatablock().getName() $= "deathVehicle")
				return;
			
			if(%this.getDamageLevel() < %this.getDatablock().maxDamage)
			{
				%atkr = %obj.client;
				%vctm = %this.client;
				%atkrSO = CityRPGData.getData(%atkr.bl_id);
				%vctmSO = CityRPGData.getData(%vctm.bl_id);
				
				if(!getWord(%atkr.valueJailData, 1))
				{
					if(CityRPG_illegalAttackTest(%atkr, %vctm))
					{
						commandToClient(%atkr, 'centerPrint', "\c6You have commited a crime. [\c3Assault\c6]", 1);
						
						if(!%atkr.getWantedLevel())
							%demerits = $CityRPG::pref::demerits::wantedLevel - %atkr.valueDemerits;
						else
							%demerits = mFloor($CityRPG::demerits::murder * (%damage / %this.getDatablock().maxDamage));
						
						CityRPG_AddDemerits(%atkr.bl_id, %demerits);
					}
				}
				else
					return;
			}
		}
		
		parent::damage(%this, %obj, %pos, %damage, %damageType);
	}
	
	function player::setScale(%this, %scale, %client)
	{
		if(isObject(%this.client))
		{
			if(CityRPGData.getData(%this.client.bl_id).valueHunger > 6)
				%scale = "1.125 1.125 1";
			else if(CityRPGData.getData(%this.client.bl_id).valueHunger == 1)
				%scale = "0.75 0.75 1";
			else
				%scale = "1 1 1";
		}
		
		parent::setScale(%this, %scale);
	}
	
	function player::setShapeNameColor(%this, %color)
	{
		if(isObject(%client = %this.client) && isObject(%client.player) && %this.getState() !$= "dead")
		{
			if(%client.getWantedLevel())
				%color = "1 0 0 1";
			else if(%client.isHost)
				%color = "0.6 0.9 1 1";
			else if(CityRPGData.getData(%client.bl_id).valueReincarnated)
				%color = "1 1 0 1";
			else
				%color = "1 1 1 1";
		}
		
		parent::setShapeNameColor(%this, %color);
	}
	
	function player::setShapeNameDistance(%this, %dist)
	{
		%dist = 24;
		
		if(isObject(%client = %this.client) && isObject(%client.player))
		{
			if(%client.getWantedLevel())
				%dist *= %client.getWantedLevel();
		}

		if(%client.isHost)
			%dist = 999;
		
		parent::setShapeNameDistance(%this, %dist);
	}
	
	// ============================================================
	// Section 4 : Misc Packages
	// ============================================================
	// Namespace Overrides
	function Armor::damage(%this, %obj, %src, %unk, %dmg, %type)
	{
		if(isObject(%obj.client.minigame) && %type == $DamageType::Vehicle)
		{
			if(%obj.client.minigame.vehicleRunOverDamage)
				parent::damage(%this, %obj, %src, %unk, %dmg, %type);
		}
		else
			parent::damage(%this, %obj, %src, %unk, %dmg, %type);
	}
	
	function Armor::onMount(%this, %obj, %veh, %slot)
	{
		parent::onMount(%this, %obj, %veh, %slot);
		
		if(isObject(keyItem))
		{
			if(%obj.client.brickGroup == %veh.brickGroup)
			{
				for(%a = 0; %a < %this.maxTools; %a++)
				{
					if(!isObject(%obj.tool[%a]) && %freeSlot $= "")
						%freeSlot = %a;
					else if(%obj.tool[%a] == nameToID(keyItem))
						%hasAKey = true;
				}
				
				if(!%hasAKey && %freeSlot !$= "")
				{
					%obj.tool[%freeSlot] = nameToID(keyItem);
					messageClient(%obj.client, 'MsgItemPickup', "", %freeSlot, nameToID(keyItem));
					schedule(50, '', "serverCmdUnUseTool", %obj.client);
				}
			}
		}
	}
	
	function WheeledVehicle::onActivate(%this, %obj, %client, %pos, %dir)
	{
		if(!%this.locked && getTrustLevel(%obj.client.brickGroup, %this.spawnBrick.getGroup()) > 0)
			parent::onActivate(%this, %obj, %client, %pos, %dir);
	}
	
	function WheeledVehicleData::onCollision(%this, %obj, %col, %pos, %vel)
	{
		if(%obj.locked && %col.getType() & $TypeMasks::PlayerObjectType && isObject(%col.client))
			commandToClient(%col.client, 'centerPrint', "\c6The vehicle is locked.", 3);
		else if(isObject(%obj.spawnBrick) && %obj.spawnBrick.getDatablock().getName() $= "CityRPGPoliceVehicleData" && isObject(%col.client) && !%col.client.getJobSO().law)
			commandToClient(%col.client, 'centerPrint', "\c6This vehicle is property of the Police Deparment.", 3);
		else
			parent::onCollision(%this, %obj, %col, %pos, %vel);
	}
	
	function itemData::onPickup(%this, %item, %obj)
	{
		parent::onPickup(%this, %item, %obj);
		
		if(isObject(%item.spawnBrick))
		{
			if(!%item.spawnBrick.getDatablock().CityRPGPermaspawn)
				%item.spawnBrick.setItem(0, ((isObject(getBrickGroupFromObject(%item.spawnBrick).client)) ? getBrickGroupFromObject(%item.spawnBrick).client : 0), true);
		}
	}
	
	function HammerImage::onHitObject(%this, %obj, %slot, %col, %pos, %normal)
	{
		if(%col.getClassName() $= "Player" && isObject(%col.client) && !%col.client.getWantedLevel())
			commandToClient(%obj.client, 'messageBoxOK', "Hey there!", "You should learn to play the RPG instead of running around hitting people upside the head with a hammer. You will enjoy yourself a lot more that way.\n\n(You have caused no damage to the person you were attacking)");
		else
			parent::onHitObject(%this, %obj, %slot, %col, %pos, %normal);
	}
	
	function KeyProjectile::onCollision(%this, %obj, %col, %fade, %pos, %normal)
	{
		parent::onCollision(%this, %obj, %col, %fade, %pos, %normal);
		
		if(%col.getDatablock().getClassName() $= "WheeledVehicleData" && mFloor(VectorLen(%col.getvelocity())) == 0)
		{
			if(getTrustLevel(%col.brickGroup, %obj.client.brickGroup) > 0)
			{
				%col.locked = !%col.locked;
				commandToClient(%obj.client, 'centerPrint', "\c6The vehicle is now \c3" @ (%col.locked ? "locked" : "unlocked") @ "\c6.", 3);
			}
			else
				commandToClient(%obj.client, 'centerPrint', "\c6The key does not fit.", 3);
		}
	}
	
	function MinigameSO::pickSpawnPoint(%mini, %client)
	{
		if(getWord(CityRPGData.getData(%client.bl_id).valueJailData, 1) > 0 && CityRPG_FindSpawn("jailSpawn"))
			%spawn = CityRPG_FindSpawn("jailSpawn");
		else
		{
			if(CityRPG_FindSpawn("personalSpawn", %client.bl_id))
				%spawn = CityRPG_FindSpawn("personalSpawn", %client.bl_id);
			else
			{
				if(CityRPG_FindSpawn("jobSpawn", CityRPGData.getData(%client.bl_id).valueJobID) && CityRPGData.getData(%client.bl_id).valueJobID != 1)
					%spawn = CityRPG_FindSpawn("jobSpawn", CityRPGData.getData(%client.bl_id).valueJobID);
				else
					%spawn = CityRPG_FindSpawn("jobSpawn", 1);
			}
		}
		
		if(%spawn)
			return %spawn;
		else
			parent::pickSpawnPoint(%mini, %client);
	}
	
	// Namespaceless Overrides
	function disconnect()
	{
		// Prevents ticks from occuring post-mission end.
		if(!$Server::Dedicated && CityRPGData.scheduleTick)
			cancel(CityRPGData.scheduleTick);
		
		parent::disconnect();
	}
	
	// Always-in-Minigame Overrides
	function miniGameCanDamage(%obj1, %obj2)
	{
		return 1;
	}
	
	function miniGameCanUse(%obj1, %obj2)
	{
		return 1;
	}
	
	function getMiniGameFromObject(%obj)
	{
		return CityRPGMini;
	}
	
	// ============================================================
	// Section 5 : Chat Functions/Packages
	// ============================================================	
	function serverCmdmessageSent(%client, %text)
	{
		if(isObject(%client.player) && isObject(%client.CityRPGTrigger) && isObject(%client.CityRPGTrigger.parent) && %client.CityRPGTrigger.parent.getDatablock().CityRPGBrickType == 2)
			%client.CityRPGTrigger.parent.getDatablock().parseData(%client.CityRPGTrigger.parent, %client, "", %text);
		else if(getWord(CityRPGData.getData(%client.bl_id).valueJailData, 1) > 0)
		{
			serverCmdteamMessageSent(%client, %text);
			return;
		}
		else
		{
			if(strLen(%text) > 10 && strcmp(strupr(%text), %text) $= "0" && strcmp(strlwr(%text), %text) $= "1")
			{
				messageClient(%client, '', "\c5Please do not type in all caps.");
				%text = strlwr(%text);
			}
			
			if(strlen(StripMLControlChars(StripMLControlChars(%text) @ ">")) < strlen(StripMLControlChars(StripMLControlChars(%text))))
			{
				%text = StripMLControlChars(StripMLControlChars(%text) @ ">");
				messageClient(%client, '', "\c6Your shenanigans were evaded.");
			}
			
			parent::serverCmdmessageSent(%client, %text);
		}
	}
	
	function serverCmdteamMessageSent(%client, %text)
	{
		%text = StripMLControlChars(%text);
		
		if(%text !$= "" && %text !$= " ")
		{
			if(getWord(CityRPGData.getData(%client.bl_id).valueJailData, 1))
			{
				for(%i = 0; %i < ClientGroup.getCount();%i++)
				{
					%subClient = ClientGroup.getObject(%i);
					if(getWord(CityRPGData.getData(%subClient.bl_id).valueJailData, 1))
					{
						messageClient(%subClient, '', "\c3[<color:777777>Inmate\c3]" SPC %client.name @ "<color:777777>:" SPC %text);
					}
				}
				echo("(Convict Chat)" SPC %client.name @ ":" SPC %text);
			}
			else
			{
				for(%i = 0;%i < ClientGroup.getCount();%i++)
				{
					%subClient = ClientGroup.getObject(%i);
					if(CityRPGData.getData(%subClient.bl_id).valueJobID == CityRPGData.getData(%client.bl_id).valueJobID && !getWord(CityRPGData.getData(%subClient.bl_id).valueJailData, 1))
					{
						messageClient(%subClient, '', "\c3[<color:" @ %client.getJobSO().tmHexColor @ ">" @ %client.getJobSO().name @ "\c3]" SPC %client.name @ "<color:" @ %client.getJobSO().tmHexColor @ ">:" SPC %text);
					}
				}
				echo("(" @ %client.getJobSO().name @ ")" SPC %client.name @ ":" SPC %text);
			}
		}
	}
	
	function serverCmdcreateMiniGame(%client)
	{
		messageClient(%client, '', "I'm afraid I can't let you do that," SPC %client.name @ ".");
	}
	
	function serverCmdleaveMiniGame(%client)
	{
		messageClient(%client, '', "I'm afraid I can't let you do that," SPC %client.name @ ".");
	}
	
	function serverCmddropTool(%client, %toolID)
	{
		if(getWord(CityRPGData.getData(%client.bl_id).valueJailData, 1) > 0)
			messageClient(%client, '', "\c6You can't drop tools while in jail.");
		else
			parent::serverCmddropTool(%client, %toolID);
	}
	
	function serverCmdsuicide(%client)
	{
		if(getWord(CityRPGData.getData(%client.bl_id).valueJailData, 1) > 0)
			commandToClient(%client, '', "\c6You cannot suicide while in jail.", 3);
		else if(%client.getWantedLevel())
		{
			for(%a = 0; %a < ClientGroup.getCount(); %a++)
			{
				%subClient = ClientGroup.getObject(%a);
				if(isObject(%subClient.player) && isObject(%client.player) && %subClient != %client)
				{
					if(VectorDist(%subClient.player.getPosition(), %client.player.getPosition()) <= 16)
					{
						if(%subClient.player.currTool > -1)
						{
							if(%subClient.player.tool[%subClient.player.currTool].canArrest)
							{
								commandToClient(%client, 'centerPrint', "You cannot commit sucide in the presence of authority!", 3);
								return;
							}
						}
					}
				}
			}
			
			parent::serverCmdsuicide(%client);
		}
		else
		{
			%client.moneyOnSuicide = CityRPGData.getData(%client.bl_id).valueMoney;
			parent::serverCmdsuicide(%client);
		}
	}

	function serverCmdUpdateBodyColors(%client, %headColor)
	{
		// The only thing we want from this command is the facial color, which determines skin color in the clothing mod.
		%client.headColor = %headColor;
		%client.applyForcedBodyColors();
	}
	
	function serverCmdUpdateBodyParts(%client)
	{
		// There is no useful information that the game could derive from UpdateBodyParts. Simply returning.
		return;
	}
	
	function wrenchImage::onHitObject(%this, %player, %a, %brick, %particle, %f)
	{
		if(%brick.getClassName() !$= "fxDtsBrick") return;

		if((%brickData = %brick.getDatablock()).CityRPGBrickType)
		{
			%player.client.centerPrint("You are not allowed to edit this type of brick.", 3);
			return;
		}

		return parent::onHitObject(%this, %player, %a, %brick, %particle, %f);
	}
};
activatePackage(CityRPG_MainPackage);