// ============================================================
// Project				:	CityRPG
// Author				:	Iban
// Description			:	Your new ScriptObject overlords.
// ============================================================
// Table of Contents
// 1. JobSO
// 2. CitySO
// 3. CalendarSO
// 4. ClothesSO
// 5. ResourceSO
// ============================================================

// ============================================================
// Section 1 : JobsSO
// ============================================================
function JobSO::populateJobs(%so)
{
	for(%a = 1; isObject(%so.job[%a]); %a++)
	{
		%so.job[%a].delete();
		%so.job[%a] = "";
	}
	
	// NOTE: Order is incredibly important. Jobs are referenced by ID, which is determined by order.
	// Mixing up the order of these professions will cause save data to reference the wrong job.
	%so.addJobFromFile("civilian");
	%so.addJobFromFile("grocer");
	%so.addJobFromFile("policeassistant");
	%so.addJobFromFile("policechief");
	%so.addJobFromFile("pickpocket");
	%so.addJobFromFile("bountyhunter");
	%so.addJobFromFile("official");
	%so.addJobFromFile("miner");
	%so.addJobFromFile("shopkeeper");
	%so.addJobFromFile("armsdealer");
	%so.addJobFromFile("lumberjack");
	%so.addJobFromFile("laborer");
	%so.addJobFromFile("tailor");
	%so.addJobFromFile("councilmember");
	%so.addJobFromFile("administrator");
	%so.addJobFromFile("criminal");
	%so.addJobFromFile("policeman");
	%so.addJobFromFile("hardenedcriminal");
	%so.addJobFromFile("shopowner");
	%so.addJobFromFile("ceo");
	%so.addJobFromFile("GangMember");
	%so.addJobFromFile("GangLeader");
	%so.addJobFromFile("ndofficer");
	%so.addJobFromFile("ndchief");
	%so.addJobFromFile("spike");
        	%so.addJobFromFile("aokicosplayer");
}

function JobSO::addJobFromFile(%so, %file)
{
	if(isFile("./jobs/" @ %file @ ".cs"))
	{
		%jobID = %so.getJobCount() + 1;
		exec("./jobs/" @ %file @ ".cs");
		%so.job[%jobID] = new scriptObject()
		{
			id		= %jobID;
			
			name		= $CityRPG::jobs::name;
			invest		= $CityRPG::jobs::initialInvestment;
			pay		= $CityRPG::jobs::pay;
			tools		= $CityRPG::jobs::tools;
			education	= $CityRPG::jobs::education;
			db		= $CityRPG::jobs::datablock;
			hostonly	= $CityRPG::jobs::hostonly;
			adminonly	= $CityRPG::jobs::adminonly;
			usepolicecars	= $CityRPG::jobs::usepolicecars;
			usecrimecars	= $CityRPG::jobs::usecrimecars;
			outfit		= $CityRPG::jobs::outfit;
			
			sellItems	= $CityRPG::jobs::sellItems;
			sellFood	= $CityRPG::jobs::sellFood;
			sellServices 	= $CityRPG::jobs::sellServices; // Unused.
			sellClothes 	= $CityRPG::jobs::sellClothes;
			
			law		= $CityRPG::jobs::law;
			canPardon	= $CityRPG::jobs::canPardon;
			
			thief		= $CityRPG::jobs::thief;
			hideJobName	= $CityRPG::jobs::hideJobName;
			
			bountyOffer	= $CityRPG::jobs::offerer;
			bountyClaim	= $CityRPG::jobs::claimer;
			
			laborer		= $CityRPG::jobs::labor;
			
			tmHexColor	= $CityRPG::jobs::tmHexColor;
			helpline	= $CityRPG::jobs::helpline;
		};
		
		if(!isObject("CityRPGJob" @ %jobID @ "SpawnBrickData"))
		{
			datablock fxDtsBrickData(CityRPGSpawnBrickData : brickSpawnPointData)
			{
				category = "CityRPG";
				subCategory = "CityRPG Spawns";
				
				uiName = %so.job[%jobID].name SPC "Spawn";
				
				specialBrickType = "";
				
				CityRPGBrickType = 3;
				CityRPGBrickAdmin = true;
				
				spawnData = "jobSpawn" SPC %jobID;
			};
			
			CityRPGSpawnBrickData.setName("CityRPGJob" @ %jobID @ "SpawnBrickData");
		}
		
		deleteVariables("$CityRPG::jobs::*");
	}
}

function JobSO::getAnAlias(%so)
{
	%jobCount = 0;
	
	for(%a = 1; isObject(%so.job[%a]); %a++)
	{
		if(!%so.job[%a].hideJobName)
		{
			%jobCount++;
			%jobName[%jobCount] = %so.job[%a].name;
		}
	}
	
	if(%jobCount)
		return %jobName[getRandom(1, %jobCount)];
	else
		return "Iban";
}

function JobSO::getJobCount(%so)
{
	for(%a = 0; isObject(%so.job[%a + 1]); %a++) { }
	return %a;
}

if(!isObject(JobSO))
{
	new scriptObject(JobSO) { };
	JobSO.populateJobs();
}

// ============================================================
// Section 2 : CitySO
// ============================================================
function CitySO::loadData(%so)
{
	if(isFile("config/server/IbanMods/CityRPG/City.cs"))
	{
		exec("config/server/IbanMods/CityRPG/City.cs");
		%so.minerals		= $CityRPG::temp::citydata::datumminerals;
		%so.lumber			= $CityRPG::temp::citydata::datumlumber;
	}
	else
	{
		%so.value["minerals"] = 0;
		%so.value["lumber"] = 0;
	}
}

function CitySO::saveData(%so)
{
	$CityRPG::temp::citydata::datum["minerals"]		= %so.minerals;
	$CityRPG::temp::citydata::datum["lumber"]		= %so.lumber;
	export("$CityRPG::temp::citydata::*", "config/server/IbanMods/CityRPG/City.cs");
}

if(!isObject(CitySO))
{
	new scriptObject(CitySO) { };
	CitySO.loadData();
}

// ============================================================
// Section 3 : CalendarSO
// ============================================================
// ============================================================
// Section 3 : CalendarSO
// ============================================================
function CalendarSO::loadCalendar(%so)
{
	// Counters
	%so.numOfMonths = 12;
	%so.zbNumMonths = %so.numOfMonths - 1;
	
	// Names
	%so.nameOfMonth[0] = "January";
	%so.nameOfMonth[1] = "February";
	%so.nameOfMonth[2] = "March";
	%so.nameOfMonth[3] = "April";
	%so.nameOfMonth[4] = "May";
	%so.nameOfMonth[5] = "June";
	%so.nameOfMonth[6] = "July";
	%so.nameOfMonth[7] = "August";
	%so.nameOfMonth[8] = "September";
	%so.nameOfMonth[9] = "October";
	%so.nameOfMonth[10] = "November";
	%so.nameOfMonth[11] = "December";
	
	// Days
	%so.daysInMonth[0] = 31;
	%so.daysInMonth[1] = 28;
	%so.daysInMonth[2] = 31;
	%so.daysInMonth[3] = 30;
	%so.daysInMonth[4] = 31;
	%so.daysInMonth[5] = 30;
	%so.daysInMonth[6] = 31;
	%so.daysInMonth[7] = 31;
	%so.daysInMonth[8] = 30;
	%so.daysInMonth[9] = 31;
	%so.daysInMonth[10] = 30;
	%so.daysInMonth[11] = 31;
	
	// Holidays
	%so.holiday[1] = "\c2Happy New Year!";
	%so.holiday[58] = "\c2Xenos109's \c6Birthday";
	%so.holiday[91] = "\c2A\c1p\c2r\c1i\c2l \c0F\c3o\c0o\c3l\c0s \c7D\c6a\c7y\c6!";
	%so.holiday[213] = "\c3It's Testing Day!";
	%so.holiday[265] = "\c1It's \c5Random\c3 Day\c4!\c2!";
	%so.holiday[350] = "\c0H\c3a\c2p\c1p\c5y\c6 Holidays\c7!";
	%so.holiday[351] = "\c6It's \c7Aoki's\c6 Birthday! You better have a present, or you will see his \c0RAGE!";
}

function CalendarSO::getDate(%so, %client)
{
	%ticks = %so.date;
	
	for(%a = 0; %ticks > %so.daysInMonth[%a % %so.numOfMonths]; %a++)
	{
		%ticks -= %so.daysInMonth[%a % %so.numOfMonths];
	}
	
	%year = mFloor(%a / %so.numOfMonths);
	
	// If the second number from last is a "1" (e.g. 12 or 516), the suffix will always be "th"
	if(strlen(%ticks) > 1 && getSubStr(%ticks, (strlen(%ticks) - 2), 1) $= "1")
	{
		%suffix = "th";
	}
	// If not, it can either be "st," "nd," "rd," or "th," depending on the last numeral.
	else
	{
		switch(getSubStr(%ticks, (strlen(%ticks) - 1), 1))
		{
			case 1: %suffix = "st";
			case 2: %suffix = "nd";
			case 3: %suffix = "rd";
			default: %suffix = "th";
		}
	}
	
	messageAll('', '\c6Today, on \c3%1\c6 the \c3%2\c6%4 of year \c3%3\c6, ...', %so.nameOfMonth[%so.getMonth()], %ticks, %year, %suffix);
	
	if(%so.holiday[%so.getCurrentDay()] !$= "")
		messageAll('', "\c6 -" SPC %so.holiday[%so.getCurrentDay()]);
		
	if(isObject(WeatherSO))
	{
		WeatherSO.todaysForecast();
	}
}

function CalendarSO::getMonth(%so)
{
	%ticks = %so.date;
	
	for(%a = 0; %ticks > %so.daysInMonth[%a % %so.numOfMonths]; %a++)
		%ticks -= %so.daysInMonth[%a % %so.numOfMonths];
	
	%month = %a % %so.numOfMonths;
	return %month;
}

function CalendarSO::dumpCalendar(%so)
{
	for(%a = 0; %so.daysInMonth[%a] !$= ""; %a++)
	{
		echo(%so.nameOfMonth[%a] SPC "has" SPC %so.daysInMonth[%a] SPC "days.");
	}
}

function CalendarSO::getYearLength(%so)
{
	for(%a = 0; %so.daysInMonth[%a] > 0; %a++)
	{
		%totalLength += %so.daysInMonth[%a];
	}
	
	return %totalLength;
}

function CalendarSO::getCurrentDay(%so)
{
	return (%so.date % %so.getYearLength());
}

function CalendarSO::loadData(%so)
{
	if(isFile("config/server/CityRPG/CityRPG/Calendar.cs"))
	{
		exec("config/server/CityRPG/CityRPG/Calendar.cs");
		%so.date = $CityRPG::temp::calendar::datumdate;
	}
	else
	{
		%so.date = 0;
	}
	
	%so.loadCalendar();
}

function CalendarSO::saveData(%so)
{
	$CityRPG::temp::calendar::datum["date"]	= %so.date;
	export("$CityRPG::temp::calendar::*", "config/server/CityRPG/CityRPG/Calendar.cs");
}

if(!isObject(CalendarSO))
{
	new scriptObject(CalendarSO) { };
	CalendarSO.schedule(1, "loadData");
}

// ============================================================
// Section 4 : ClothesSO
// ============================================================
function ClothesSO::loadClothes(%so)
{
	// Clothing Data
	%so.color["none"]		= "1 1 1 1";
	%so.node["none"]		= "0";
	
	// Outfits
	// Outfits use index instead of names.
	// Do not repeat indexes.
	// This is the order they appear in the GUI.
	%so.str[1]	= "none none none none whitet whitet skin bluejeans blackshoes default default";
	%so.uiName[1]	= "Default";
	%so.sellName[1]	= "Default Suit";

	%so.str[2]	= "none brownhat keep keep greenshirt greenshirt keep greyPants blackshoes default default";
	%so.uiName[2]	= "Basic";
	%so.sellName[2]	= "Basic Outfit";
	
	%so.str[3]	= "keep skullcap keep keep blackshirt blackshirt blackgloves blackPants blackshoes default default";
	%so.uiName[3]	= "Gimp";
	%so.sellName[3]	= "Gimp Suit";
	
	%so.str[4]	= "none none none none whitet redsleeve keep brightbluePants blueshoes default default";
	%so.uiName[4]	= "Blockhead";
	%so.sellName[4]	= "Blockhead Clothes";

	%so.str[5]	= "keep keep keep keep greenshirt greenshirt keep brownPants blackshoes default worm-sweater";
	%so.uiName[5]	= "Nerd";
	%so.sellName[5]	= "Nerd Suit";

	%so.str[6]	= "keep keep keep keep blackshirt blackshirt keep blackPants blackshoes default Mod-Suit";
	%so.uiName[6]	= "Business";
	%so.sellName[6]	= "Business Suit";

	%so.str[7]	= "keep keep keep keep blueshirt blueshirt keep bluePants blackshoes default Mod-Suit";
	%so.uiName[7]	= "Council";
	%so.sellName[7]	= "Council Suit";

	%so.str[8]	= "keep keep keep keep skingen skingen skingen skingen skingen default default";
	%so.uiName[8]	= "Naked";
	%so.sellName[8]	= "B-Day Suit";

	// Hats
	%so.color["brownhat"]	= "0.329 0.196 0.000 1.000";
	%so.node["brownhat"]	= "4";
	%so.str["brownhat"]	= "keep this keep keep keep keep keep keep keep";
	
	%so.color["piratehat"]	= "0.078 0.078 0.078 1";
	%so.node["piratehat"]	= "5";
	%so.str["piratehat"]	= "keep this keep keep keep keep keep keep keep";
	
	%so.color["copHat"]	= "0 0.141176 0.333333 1";
	%so.node["copHat"]	= "6";
	%so.str["copHat"]	= "keep this keep keep keep keep keep keep keep";
	
	%so.color["skullcap"]	= "0.200 0.200 0.200 1.000";
	%so.node["skullcap"]	= "7";
	%so.str["skullcap"]	= "keep this keep keep keep keep keep keep keep";
	
	%so.color["aokicap"]	= "0.990654 0.962617 0.686916 1";
	%so.node["aokicap"]	= "7";
	%so.str["aokicap"]	= "keep this keep keep keep keep keep keep keep";
	
	%so.color["ndHat"]	= "0.200 0.200 0.200 1.000";
	%so.node["ndHat"]	= "6";
	%so.str["ndHat"]	= "keep this keep keep keep keep keep keep keep";
	
	// Gloves
	%so.color["blackgloves"] = "0.200 0.200 0.200 1.000";
	%so.node["blackgloves"]	= "0";
	%so.str["blackgloves"]	= "keep keep keep keep keep keep this keep keep";
	
	// Shirts
	%so.color["pinkt"]	= "1 0.75 0.79 1";
	%so.node["pinkt"]	= "gender";
	%so.str["pinkt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["aokishirt"]	= "0.621495 0.859813 0.976636 1";
	%so.node["aokishirt"]	= "gender";
	%so.str["aokishirt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["aokisleeve"]	= "0.934579 0.257009 0.266355 1";
	%so.node["aokisleeve"]	= "gender";
	%so.str["aokisleeve"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["pinktank"]	= "1 0.07 0.57 1";
	%so.node["pinktank"]	= "gender";
	%so.str["pinktank"]	= "keep keep keep keep this skingen keep keep keep";
	
	%so.color["whitet"]	= "1 1 1 1";
	%so.node["whitet"]	= "gender";
	%so.str["whitet"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["copShirt"]	= "0 0.141176 0.333333 1";
	%so.node["copShirt"]	= "gender";
	%so.str["copShirt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["jumpsuit"]	= "1 0.617 0 1";
	%so.node["jumpsuit"]	= "gender";
	%so.str["jumpsuit"]	= "keep keep keep keep this this keep this this";
	
	%so.color["blackshirt"]	= "0.200 0.200 0.200 1.000";
	%so.node["blackshirt"]	= "gender";
	%so.str["blackshirt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["brownshirt"]	= "0.329 0.196 0.000 1.000";
	%so.node["brownshirt"]	= "gender";
	%so.str["brownshirt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["greenshirt"]	= "0.00 0.262 0.00 1";
	%so.node["greenshirt"]	= "gender";
	%so.str["greenshirt"]	= "keep keep keep keep this this keep keep keep";

	%so.color["blueshirt"]	= "0.0 0.141 0.333 1";
	%so.node["blueshirt"]	= "gender";
	%so.str["blueshirt"]	= "keep keep keep keep this this keep keep keep";

	// Pants
	%so.color["bluejeans"]	= "0 0.141 0.333 1";
	%so.node["bluejeans"]	= "0";
	%so.str["bluejeans"]	= "keep keep keep keep keep keep keep this keep";
	
	%so.color["aokipants"]	= "0.621495 0.859813 0.976636 1";
	%so.node["aokipants"]	= "0";
	%so.str["aokipants"]	= "keep keep keep keep keep keep keep this keep";
	
	%so.color["blackPants"] = "0.200 0.200 0.200 1.000";
	%so.node["blackPants"]	= "0";
	%so.str["blackPants"]	= "keep keep keep keep keep keep keep this keep";
	
	%so.color["brownPants"] = "0.329 0.196 0.000 1.000";
	%so.node["brownPants"]	= "0";
	%so.str["brownPants"]	= "keep keep keep keep keep keep keep this keep";
	
	%so.color["greyPants"] = "0.300 0.300 0.300 1.000";
	%so.node["greyPants"]	= "0";
	%so.str["greyPants"]	= "keep keep keep keep keep keep keep this keep";

	%so.color["brightbluePants"] = "0.200 0.000 0.800 1.000";
	%so.node["brightbluePants"]	= "0";
	%so.str["brightbluePants"]	= "keep keep keep keep keep keep keep this keep";

	%so.color["bluePants"] = "0.0 0.141 0.333 1";
	%so.node["bluePants"]	= "0";
	%so.str["bluePants"]	= "keep keep keep keep keep keep keep this keep";

	// Shoes
	%so.color["blackshoes"]	= "0.200 0.200 0.200 1.000";
	%so.node["blackshoes"]	= "0";
	%so.str["blackshoes"]	= "keep keep keep keep keep keep keep keep this";

	%so.color["aokishoes"]	= "0.121495 0.598131 0.831776 1";
	%so.node["aokishoes"]	= "0";
	%so.str["aokishoes"]	= "keep keep keep keep keep keep keep keep this";

	%so.color["brownshoes"]	= "0.329 0.196 0.000 1.000";
	%so.node["brownshoes"]	= "0";
	%so.str["brownshoes"]	= "keep keep keep keep keep keep keep keep this";

	%so.color["blueshoes"]	= "0.200 0.000 0.800 1.000";
	%so.node["blueshoes"]	= "0";
	%so.str["bluehoes"]	= "keep keep keep keep keep keep keep keep this";

	//Misc
	%so.color["redsleeve"] = "0.900 0.220 0.000 1.000";
	%so.node["redsleeve"] = "gender";
	%so.color["redsleeve"] = "keep keep keep keep this this keep keep keep";
}

function ClothesSO::postEvents(%so)
{
	%str = "list";
	
	for(%a = 1; %so.str[%a] !$= ""; %a++)
		%str = %str SPC %so.uiName[%a] SPC %a;
	
	if(%str !$= "")
	{
		registerOutputEvent("fxDTSBrick", "sellClothes", %str TAB "int 0 500 1");
		
		for(%b = 0; %b < ClientGroup.getCount(); %b++)
		{
			%subClient = ClientGroup.getObject(%b);
			serverCmdRequestEventTables(%subClient);
			messageClient(%subClient, '', "\c6Your Event Tables have been updated. If you do not know what that means, ignore this message.");
		}
	}
}

function ClothesSO::getColor(%so, %client, %item)
{
	if(%item $= "skin" || %item $= "skingen")
		return %client.headColor;
	else
	{
		%color = %so.color[%item];
		return %color;
	}
}

function ClothesSO::getNode(%so, %client, %item)
{
	if(%item $= "skin")
		return 0;
	else if(%item $= "skingen")
		return (%client.gender $= "Male" ? 0 : 1);
	else
	{
		%node = %so.node[%item];
	
		if(%node $= "gender" || %node $= "skingen")
			return (%client.gender $= "Male" ? 0 : 1);
		else
			return %node;
	}
}

function ClothesSO::getDecal(%so, %client, %segment, %item)
{
	if(%item $= "" || %item $= "default")
	{
		if(%segment $= "face")
			return "smiley";
		else if(%segment $= "chest")
			return "AAA-none";
	}
	else
		return %item;
}

function ClothesSO::giveItem(%so, %client, %item)
{
	if(strLen(%so.str[%item]) && isObject(%client))
	{
		%outfit = CityRPGData.getData(%client.bl_id).valueOutfit;
		
		for(%a = 0; %a < getWordCount(%outfit); %a++)
		{
			if(getWord(%so.str[%item], %a) $= "keep")
				%newOutfit = (%newOutfit $= "" ? getWord(%outfit, %a) : %newOutfit SPC getWord(%outfit, %a));
			else if(getWord(%so.str[%item], %a) $= "this")
				%newOutfit = (%newOutfit $= "" ? %item : %newOutfit SPC %item);
			else
				%newOutfit = (%newOutfit $= "" ? getWord(%so.str[%item], %a) : %newOutfit SPC getWord(%so.str[%item], %a));
		}
		
		CityRPGData.getData(%client.bl_id).valueOutfit = %newOutfit;
		%client.applyBodyParts();
		%client.applyBodyColors();
	}
}

if(!isObject(ClothesSO))
{
	new scriptObject(ClothesSO) { };
	ClothesSO.schedule(1, "loadClothes");
	ClothesSO.schedule(1, "postEvents");
}

// ============================================================
// Section 5 : ResourceSO
// ============================================================

function ResourceSO::addResources(%this)
{
	//makes sure it's clear when this is re-exec'd
	for(%a = 0; isObject(ResourceSO.resource[%a]); %a++)
	{
		ResourceSO.resource[%a].delete();
		ResourceSO.resource[%a] = "";
	}

	ResourceSO.mineralCount = 6;
	ResourceSO.treeCount = 3;

	ResourceSO.mineral[1] = new scriptObject() {
		id = 1;
		name = "Dirt";
		totalHits = 10;
		BPH = 0.21;
		color = "0.60 0.20 0.20 1";
		respawnMultiplier = 2.5; };

	ResourceSO.mineral[2] = new scriptObject() {
		id = 2;
		name = "Tin";
		totalHits = 15;
		BPH = 0.21;
		color = "0.6 0.6 0.6 1";
		respawnMultiplier = 1; };

	ResourceSO.mineral[3] = new scriptObject() {
		id = 3;
		name = "Copper";
		totalHits = 30;
		BPH = 0.21;
		color = "1 0.45 0.2 1";
		respawnMultiplier = 1; };

	ResourceSO.mineral[4] = new scriptObject() {
		id = 4;
		name = "Iron";
		totalHits = 50;
		BPH = 0.21;
		color = "0.5 0.5 0.5 1";
		respawnMultiplier = 1; };

	ResourceSO.mineral[5] = new scriptObject() {
		id = 5;
		name = "Silver";
		totalHits = 65;
		BPH = 0.21;
		color = "0.9 0.9 0.9 1";
		respawnMultiplier = 1; };

	ResourceSO.mineral[6] = new scriptObject() {
		id = 6;
		name = "Gold";
		totalHits = 75;
		BPH = 0.22;
		color = "0.9 0.9 0 1";
		respawnMultiplier = 1; };

	ResourceSO.tree[1] = new scriptObject() {
		id = 1;
		name = "Pine";
		BPH = 0.21;
		TotalHits = 20;
		 Color = "0 .5 .25 1"; };

	ResourceSO.tree[2] = new scriptObject() {
		id = 2;
		name = "Dead";
		BPH = 0.21;
		TotalHits = 20;
		 Color = "0.5 0.2 0.2 1"; };

	ResourceSO.tree[3] = new scriptObject() {
		id = 3;
		name = "Birch";
		BPH = 0.21;
		TotalHits = 40;
		Color = "1 1 1 1"; };
}

if(isObject(ResourceSO))
	ResourceSO.delete();

if(!isObject(ResourceSO))
{
	new scriptObject(ResourceSO) { };
	ResourceSO.addResources();
}
